/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "chunk.h"

void chunkLoad(chunk_t *chunk, tilemap_t *tilemap,
  int32_t x, int32_t y, int32_t z
){
  int32_t tx, ty, tz, i;
  tileid_t tileId;
  tiledef_t *tileDef;
  tilesetdiv_t *div;
  char *buffer, *seek, *current;
  char fileName[128];

  // Determine file name of the chunk.
  fileName[0] = '\0';
  strcat(fileName, "world/chunks/%d_%d_%d.txt");
  sprintf(fileName, fileName, x, y, z);

  // Load contents of chunk
  buffer = assetStringLoad(fileName);
  if(buffer == NULL) return;
  seek = buffer;
  
  // Determine the size of our primitive.
  int32_t indiceCount = 0, verticeCount = 0;
  for(i = 0; i < CHUNK_TILE_COUNT; i++) {
    // Now load from the buffer.
    current = strtok_r(seek, ";", &seek);
    *(chunk->tiles + i) = current[0] - CHUNK_TILE_LOAD_ASCII;
    tileId = *(chunk->tiles + i);
    if(tileId == TILE_NULL) continue;

    // Increment tile definition.
    tileDef = tilemap->tileDefinitions+tileId;
    verticeCount += tileDef->verticeCount;
    indiceCount += tileDef->indiceCount;
  }

  // Now we know how big the primitive needs to be!
  free(buffer);
  if(indiceCount == 0) return;
  chunk->primitive = primitiveCreate(verticeCount, indiceCount);

  // For each tile
  i = 0;
  verticeCount = 0, indiceCount = 0;
  for(tx = 0; tx < CHUNK_WIDTH; tx++) {
    for(ty = 0; ty < CHUNK_HEIGHT; ty++) {
      for(tz = 0; tz < CHUNK_DEPTH; tz++) {
        // Get tile for position...
        tileId = *(chunk->tiles + (i++));
        if(tileId == TILE_NULL) continue;

        // Get the definition.
        tileDef = tilemap->tileDefinitions+tileId;
        if(tileDef->tileRender == NULL) continue;

        // Render the tile's primitive
        tileDef->tileRender(tileId, tilemap, tileDef,
          chunk->primitive, verticeCount, indiceCount,
          tx, ty, tz
        );

        // Prepare for the next render.
        verticeCount += tileDef->verticeCount;
        indiceCount += tileDef->indiceCount;
      }
    }
  }
}

void chunkUnload(chunk_t *chunk) {
  // Load chunks to zero. TODO: Necessary?
  memset(chunk->tiles, TILE_NULL, CHUNK_TILE_COUNT);

  // Unload the primitive
  if(chunk->primitive == NULL) return;
  primitiveDispose(chunk->primitive);
  chunk->primitive = NULL;
}