/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "tile.h"

tilemap_t * tileMapCreate(texture_t *texture) {
  tilemap_t *tilemap = malloc(sizeof(tilemap_t));
  if(tilemap == NULL) return NULL;
  
  tilemap->texture = texture;

  // Create the tileset for the texture.
  tilemap->tileset = tilesetCreate(
    texture->width/TILE_WIDTH, texture->height/TILE_HEIGHT,
    texture->width,            texture->height,
    0, 0, 0, 0
  );
  if(tilemap->tileset == NULL) {
    free(tilemap);
    return NULL;
  }

  // Setup the tile definitions for each tile within the tileset.
  tilemap->tileDefinitions = calloc(tilemap->tileset->count, sizeof(tiledef_t));
  if(!tilemap->tileDefinitions) {
    tilesetDispose(tilemap->tileset);
    free(tilemap);
    return NULL;
  }

  return tilemap;
}

void tileMapDispose(tilemap_t *tilemap) {
  free(tilemap->tileDefinitions);
  tilesetDispose(tilemap->tileset);
  free(tilemap);
}

void tileQuadRender(tileid_t id, tilemap_t *tilemap, tiledef_t *tileDef,
  primitive_t *primitive, int32_t verticeStart, int32_t indiceStart,
  int32_t x, int32_t y, int32_t z
) {
  tilesetdiv_t *div = tilemap->tileset->divisions + id;
  quadBuffer(primitive, z,
      x,   y, div->x0, div->y0,
    x+1, y+1, div->x1, div->y1,
    verticeStart, indiceStart
  );
}